/************************************************************************/
/*                                                                      */
/*  OMF_Load.h : Header pour le chargement/dcodage des fichiers OMF.   */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  Avril 2013  */
/************************************************************************/

#define SEGMENT_HEADER_MIN_SIZE    44       /* Taille de la partie fixe du Header */
#define SEGMENT_HEADER_SIZE        64       /* Taille habituelle (pourait tre plus grande) */

struct omf_segment_header
{
  DWORD FileOffset;            /* Offset depuis le dbut du fichier */
  DWORD SegmentOffset;         /* Offset depuis le dbut du Segment */

  DWORD ByteCnt;               /* Segment Header size + Segment Body size */
  DWORD ResSpc;                /* Number of 0x00  ajouter  la fin du Body */
  DWORD Length;                /* Taille du Segmeznt en mmoire */
  BYTE undefine_1;
  BYTE LabLen;                 /* Longueur des Noms : 10 */
  BYTE NumLen;                 /* Taille des nombres = 4 Bytes */
  BYTE Version;                /* OMF version : 2 */
  DWORD BankSize;              /* Taille d'un Bank : 64 KB si Code, entre 0 et 64 KB pour Data */
  WORD Kind;                   /* Type de Segment */
  BYTE undefine_2;
  BYTE undefine_3;
  DWORD Org;
  DWORD Align;                 /* Alignment : 0, 256 ou 64 KB */
  BYTE NumSEx;                 /* Little Endian : 0 pour le IIgs */
  BYTE undefine_4;
  WORD SegNum;                 /* Numro du Segment : 1->N */
  DWORD EntryPointOffset;      /* Point d'entre dans le Segment */
  WORD DispNameOffset;         /* Offset o se situe le LoadName */
  WORD DispDataOffset;         /* Offset o commence le Segment Body */

  int TempOrgLength;           /* Usually nothing here */
  unsigned char TempOrg[2048];

  char LoadName[256];          /* Tjs 10 de long */
  char SegName[256];           /* LabLen de long */
};


struct omf_segment
{
  struct omf_segment_header header;    /* Segment Header */

  int nb_record;                       /* Segment Body : Record List */
  struct omf_body_record *first_record;
  struct omf_body_record *last_record;

  /* Rsum des Record */
  int lconst_length;
  unsigned char *lconst_data;

  int nb_reloc;                        /* Relocation intra-segment */
  struct omf_reloc **tab_reloc;

  int nb_interseg;                     /* Relocation inter-segment */
  struct omf_interseg **tab_interseg;

  struct omf_segment *next;
};


struct omf_file
{
  char *file_path;
  char *file_name;

  int data_length;
  unsigned char *data;

  int nb_segment;
  struct omf_segment *first_segment;
  struct omf_segment *last_segment;
};

struct omf_file *LoadOMFFile(char *);
void mem_free_omf(struct omf_file *);

/************************************************************************/